const Ci = Components.interfaces;
const Cc = Components.classes;

if (typeof CuckooOptions == 'undefined') {
	var CuckooOptions = {};
	Components.utils.import("resource://cuckoo/modules/ct.jsm");
	Components.utils.import("resource://app/jsmodules/sbLibraryUtils.jsm");
	Components.utils.import("resource://app/jsmodules/sbProperties.jsm");
}

CuckooOptions.Controller = {
	_mediaTab: null,
	_mediaArray: null,

	onLoad: function() {
		// initialization code
		this._initialized = true;
		var controller = this;
		this._strings = document.getElementById("cuckoo-strings");

		// prepare data from medialist for toc
		if (typeof(gBrowser) == "undefined")
			var gBrowser = Cc["@mozilla.org/appshell/window-mediator;1"]
				.getService(Ci.nsIWindowMediator)
				.getMostRecentWindow("Songbird:Main").window.gBrowser;
		if (typeof(gBrowser.mediaTab) == "undefined" || typeof(gBrowser.mediaTab.mediaListView) == "undefined") {
			alert(this._strings.getString("noMediaListViewFound"));
			return;
		}
		this._mediaTab = gBrowser.mediaTab;
		var enumerator = new LibraryUtils.MediaListEnumeratorToArray();
		gBrowser.mediaTab.mediaListView.mediaList.enumerateAllItems(enumerator, 0);
		if (!enumerator.array.length) {
			alert(this._strings.getString("noItemInPlaylistFound"));
			return;
		}
		this._mediaArray = enumerator.array;

		// prepare dialog helper functions
		document.getElementById("export-find-button").addEventListener("command", function() { controller._doDirFind("openExportDirectory", "extensions.cuckoo.export"); }, false);
		document.getElementById("cover-cdlabelgen-find-button").addEventListener("command", function() { controller._doFileFind("openCdlabelgenLocation", "extensions.cuckoo.cover.cdlabelgen"); }, false);
		document.getElementById("cover-background-find-button").addEventListener("command", function() { controller._doEPSFind("backgroundFind", "extensions.cuckoo.cover.background"); }, false);
		document.getElementById("cover-logo-find-button").addEventListener("command", function() { controller._doEPSFind("logoFind", "extensions.cuckoo.cover.logo"); }, false);
		document.getElementById("cover-revert-button").addEventListener("command", function() { controller._doTocReset(); }, false);
		document.documentElement.getButton("extra1").addEventListener("command", function() { controller.doAbout(); }, false);
		document.documentElement.getButton("extra2").addEventListener("command", function() { controller.doExport(); }, false);

		// set calculatet values
		this._doTocReset();
	},

	onUnLoad: function() {
		this._initialized = false;
	},

	doAbout: function() {
		var url = "chrome://cuckoo/content/about.xul"
		var features = "chrome,titlebar,toolbar,centerscreen,modal";
		window.openDialog(url, "About", features);
	},

	doExport: function() {
		var playlist = Application.prefs.get("extensions.cuckoo.playlist").value;
		var expfile = Application.prefs.get("extensions.cuckoo.playlist.exported").value;
		var copy = Application.prefs.get("extensions.cuckoo.files").value;
		if (playlist && expfile && !copy) {

			alert(this._strings.getString("noFileExportButFileM3U"));
			return;
		}
		try {
			var exportDir = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
			exportDir.initWithPath(Application.prefs.get("extensions.cuckoo.export").value);
			var aFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
			var bFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);

			// cleanup target directory
			if (Application.prefs.get("extensions.cuckoo.export.wipedir").value) {
				aFile.initWithPath(exportDir.path);
				var entries = aFile.directoryEntries;
				var array = [];
				while(entries.hasMoreElements()) {
					var entry = entries.getNext();
					entry.QueryInterface(Ci.nsIFile);
					array.push(entry);
				}
				for(var i = 0; i < array.length; i++) {
					array[i].remove(true);
				}
			}

			// export filesnames and info to cdlabelgen
			if (Application.prefs.get("extensions.cuckoo.cover").value) {
				var args = new Array();
				args.push("-c"); args.push(document.getElementById("cover-title-textbox").value);
				if (document.getElementById("cover-subtitle-textbox").value != "") {
					args.push("-s"); args.push(document.getElementById("cover-subtitle-textbox").value);
				}
				if (Application.prefs.get("extensions.cuckoo.cover.type").value != "") {
					args.push(Application.prefs.get("extensions.cuckoo.cover.type").value);
				}
				if (Application.prefs.get("extensions.cuckoo.cover.omittitlecover").value)
					args.push("--no-cover-plaque");
				if (Application.prefs.get("extensions.cuckoo.cover.omittitletray").value)
					args.push("--no-tray-plaque");
				if (Application.prefs.get("extensions.cuckoo.cover.omitdate").value)
					args.push("--no-date");
				if (Application.prefs.get("extensions.cuckoo.cover.background").value != "") {
					args.push("--cover-image"); args.push(Application.prefs.get("extensions.cuckoo.cover.background").value);
					args.push("--cover-image-scaleratio"); args.push("0");
				}
				if (Application.prefs.get("extensions.cuckoo.cover.logo").value != "") {
					args.push("--tray-image"); args.push(Application.prefs.get("extensions.cuckoo.cover.logo").value);
				}
				if (Application.prefs.get("extensions.cuckoo.cover.tocexport").value) {
					var itemsFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
					itemsFile.initWithPath(exportDir.path);
					itemsFile.append(this._mediaTab.mediaListView.mediaList.name + ".toc");
					itemsFile.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, 0666);
					var foStream = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);
					foStream.init(itemsFile, 0x02 | 0x08 | 0x20, 0666, 0);
					var converter = Cc["@mozilla.org/intl/converter-output-stream;1"].createInstance(Ci.nsIConverterOutputStream);
					converter.init(foStream, "UTF-8", 0, 0);
					converter.writeString(document.getElementById("cover-toc-textbox").value);
					converter.close();
					args.push("--items-from-file"); args.push(itemsFile.path);
				} else {
					var lines = document.getElementById("cover-toc-textbox").value.split("\n");
					args.push("--items"); args.push(lines.join("%"));
				}
				var outputFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
				outputFile.initWithPath(exportDir.path);
				outputFile.append(this._mediaTab.mediaListView.mediaList.name + ".ps");
				args.push( "--output-file"); args.push(outputFile.path);

				var cdlabelgenFile = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
				cdlabelgenFile.initWithPath(Application.prefs.get("extensions.cuckoo.cover.cdlabelgen").value);

				try {
					var process = Cc["@mozilla.org/process/util;1"]
						.createInstance(Ci.nsIProcess);
					process.init(cdlabelgenFile);
					process.run(true, args, args.length);
				} catch (e) {
					CT.cs.logStringMessage(e);
				}
			}

			// export contents of current sb-playlist treeview
			if (Application.prefs.get("extensions.cuckoo.view").value) {
				var tree = this._mediaTab.mediaPage._playlist.tree;
				var exportCols = new Array();
				var exportElements = new Array();
				var col = tree.columns.getFirstColumn();
				var msg = "";
				while ( col ) {
					if (col.width != 0) {
						exportCols.push( col );
						exportElements.push( col.element.getAttribute("label") );
					}
					col = col.getNext();
				}
				var line = exportElements.join("\t");
				var list = line + "\n";
				for (var i = 0; i < tree.view.rowCount; i++) {
					exportElements = new Array(exportCols.length);
					for (var j = 0; j < exportCols.length; j++) {
						exportElements[j] = tree.view.getCellText(i,exportCols[j]);
					}
					line = exportElements.join("\t");
					list = list + line + "\n";
				}
				if (Application.prefs.get("extensions.cuckoo.view.clipboard").value) {
					// export list variable into clipboard
					var chs = Cc["@mozilla.org/widget/clipboardhelper;1"]
						.getService(Ci.nsIClipboardHelper);
					chs.copyString(list);
				} else {
					// export list variable into a file named like the playlist
					aFile.initWithPath(exportDir.path);
					aFile.append(this._mediaTab.mediaListView.mediaList.name + ".txt");
					var foStream = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);
					foStream.init(aFile, 0x02 | 0x08 | 0x20, 0666, 0);
					var converter = Cc["@mozilla.org/intl/converter-output-stream;1"].createInstance(Ci.nsIConverterOutputStream);
					converter.init(foStream, "UTF-8", 0, 0);
					converter.writeString(list);
				}
			}

			var playlist = "";
			var prefixed = Application.prefs.get("extensions.cuckoo.files.prefixed").value;

			aFile.initWithPath(exportDir.path);
			for (var i = 0; i < this._mediaArray.length; i++) {
				var item = this._mediaArray[i];
				var contentURL = item.getProperty(SBProperties.contentURL);
				var trackName = item.getProperty(SBProperties.trackName);
				if (typeof contentURL != 'undefined'
					&& typeof trackName != 'undefined') {
					var outName = "";
					if (prefixed) {
						outName = CT.prefixNumber(i,'0',4) + " - " + trackName;
					} else {
						outName = trackName;
					}
					var url = CT.ios.newURI(contentURL, null, null);
					if (!url || !url.schemeIs("file")) throw "Expected a file URL.";
					var inFile = url.QueryInterface(Ci.nsIFileURL).file;
					var pos = inFile.leafName.lastIndexOf('.');
					if (pos > -1) {
						outName = outName + inFile.leafName.substr(pos);
					}
					bFile.initWithPath(aFile.path);
					bFile.append(outName);
					playlist += ( expfile ? bFile.path : inFile.path ) + "\n";
					if (copy) {
						inFile.copyTo(aFile, outName);
					}
				}
			}

			// export playlist
			if (Application.prefs.get("extensions.cuckoo.playlist").value) {
				aFile.initWithPath(exportDir.path);
				aFile.append(this._mediaTab.mediaListView.mediaList.name + ".m3u");
				var foStream = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);
				foStream.init(aFile, 0x02 | 0x08 | 0x20, 0666, 0);
				var converter = Cc["@mozilla.org/intl/converter-output-stream;1"].createInstance(Ci.nsIConverterOutputStream);
				converter.init(foStream, "UTF-8", 0, 0);
				converter.writeString(playlist);
			}
		} catch (e) {
			CT.cs.logStringMessage(e);
		}
	},

	_doTocReset: function() {
		var toc = "";
		for (var i = 0; i < this._mediaArray.length; i++) {
			var item = this._mediaArray[i];
			var trackName = item.getProperty(SBProperties.trackName);
			var artistName = item.getProperty(SBProperties.artistName);
			var duration = CT.ssToMM_SS(item.getProperty(SBProperties.duration));
			var j = i + 1;
			toc += j + " " + trackName + " - " + artistName + "    " + duration + "\n";
		}
		document.getElementById("cover-title-textbox").value = this._mediaTab.mediaListView.mediaList.name;
		document.getElementById("cover-subtitle-textbox").value =  "";
		document.getElementById("cover-toc-textbox").value =  toc;
	},

	_doFileFind: function(aTitleString,aPrefName) {
		var fp = Cc["@mozilla.org/filepicker;1"].createInstance(Ci.nsIFilePicker);
		fp.init(window, this._strings.getString(aTitleString), Ci.nsIFilePicker.modeOpen);
		fp.appendFilters(Ci.nsIFilePicker.filterAll);
		var rv = fp.show();
		if (rv == Ci.nsIFilePicker.returnOK || rv == Ci.nsIFilePicker.returnReplace) {
			Application.prefs.setValue(aPrefName, fp.file.path);
		}
	},

	_doDirFind: function(aTitleString,aPrefName) {
		var fp = Cc["@mozilla.org/filepicker;1"].createInstance(Ci.nsIFilePicker);
		fp.init(window, this._strings.getString(aTitleString), Ci.nsIFilePicker.modeGetFolder);
		fp.appendFilters(Ci.nsIFilePicker.filterAll);
		var rv = fp.show();
		if (rv == Ci.nsIFilePicker.returnOK || rv == Ci.nsIFilePicker.returnReplace) {
			Application.prefs.setValue(aPrefName, fp.file.path);
		}
	},

	_doEPSFind: function(aTitleString,aPrefName) {
		var fp = Cc["@mozilla.org/filepicker;1"].createInstance(Ci.nsIFilePicker);
		fp.init(window, this._strings.getString(aTitleString), Ci.nsIFilePicker.modeOpen);
		fp.appendFilter("Encapsulated postscript", "*.eps");
		var rv = fp.show();
		if (rv == Ci.nsIFilePicker.returnOK || rv == Ci.nsIFilePicker.returnReplace) {
			Application.prefs.setValue(aPrefName, fp.file.path);
		}
	},

};

window.addEventListener("load", function(e) { CuckooOptions.Controller.onLoad(e); }, false);
window.addEventListener("unload", function(e) { CuckooOptions.Controller.onUnLoad(e); }, false);
